import Combine
import Foundation

// MARK: Private action processing
extension XMPPService {
    func _doAction(_ action: Action) {
        // actions with no client JID
        if case .deinitClient = action.type {
            // TODO: check that client disappears from memory
            clients.removeValue(forKey: action.jid)
        }
        if case .initClient = action.type {
            let client = XMPPClient(jid: action.jid, updateCallback: clientUpdated)
            clients[action.jid] = client
            feed.send(.init(jid: action.jid, eventType: .clientInitialised))
        }

        // actions with client JID
        guard let client = clients[action.jid] else {
            feed.send(.init(jid: action.jid, eventType: .serviceError(.unknownClient)))
            return
        }
        switch action.type {
        case .initClient:
            break

        case .deinitClient:
            break

        case .connectToServer:
            client.reconnect()

        case .login(let password):
            client.loginWith(password)
        }
    }
}
